@@ -2,11 +2,11 @@
 
 ;; Author: Johannes Mueller <github@johannes-mueller.org>
 ;; URL: https://github.com/johannes-mueller/test-cockpit.el
-;; Package-Version: 0.1.0
-;; Package-Revision: 75711b58b9d0
+;; Package-Version: 20260204.1724
+;; Package-Revision: e8bb43adf1e7
 ;; License: GPLv3
 ;; SPDX-License-Identifier: GPL-3.0-only
-;; Package-Requires: ((emacs "28.1") (projectile "2.7") (toml "20230411.1449"))
+;; Package-Requires: ((emacs "28.1") (projectile "2.7") (toml "0.2.0"))
 
 ;;; Commentary:
 
@@ -104,8 +104,8 @@ a derived class of `test-cockpit--engine'.")
               :initform nil)
    (last-interactive-cmd :initarg :last-interactive-cmd
                          :initform nil)
-   (is-dummy-engine :initarg :is-dummy-engine
-                    :initform nil))
+   (project-type :initarg :project-type
+                 :initform nil))
   "The base class for a test-cockpit engine.
 For every project type supported by `test-cockpit.el' a derived
 class is needed which implements the methods of the base class.")
@@ -153,8 +153,11 @@ the argument list passed to the test frame work."
 PROJECT-TYPE is the type given by `pojectile-project-type' and
 ENGINE-CLASS is a derived class of `test-cockpit-engine'."
   (setq test-cockpit--project-types
-        (cons `(,project-type . (lambda () (make-instance ,engine-class)))
-              test-cockpit--project-types)))
+        (cons `(,project-type . (lambda () (make-instance ,engine-class :project-type ,(symbol-name project-type))))
+              test-cockpit--project-types))
+  (test-cockpit-register-project-type-alias project-type project-type))
+
+(defvar test-cockpit--project-type-aliases nil)
 
 (defun test-cockpit-register-project-type-alias (alias project-type)
   "Register an alias for a known project type.
@@ -162,22 +165,25 @@ Some project types are similar in a way that they can be tested
 by the same commands, yet they are different for projectile.  In
 those cases the already registered PROJECT-TYPE can be registered
 again as ALIAS."
-  (setq test-cockpit--project-types
-        (cons `(,alias . ,(alist-get project-type test-cockpit--project-types))
-              test-cockpit--project-types)))
+  (push (cons alias project-type) test-cockpit--project-type-aliases))
+
+(defun test-cockpit--primary-project-type (&optional project-type)
+  "Return the primary project type for the PROJECT-TYPE or detected project type."
+  (alist-get (or project-type (projectile-project-type))
+             test-cockpit--project-type-aliases nil nil 'equal))
 
 (defun test-cockpit--make-dummy-engine ()
   "Make a dummy for the case that the project type is not supported.
 In those cases we fall back to `porjectile-test-project'."
-  (make-instance 'test-cockpit--engine :is-dummy-engine t))
+  (make-instance 'test-cockpit--engine))
 
 (defun test-cockpit--make-engine ()
   "Make a new engine for the current project type.
 If the current project type is not supported a dummy engine is
 returned."
-  (if-let* ((engine-factory (alist-get (projectile-project-type) test-cockpit--project-types))
+  (if-let* ((engine-factory (alist-get (test-cockpit--primary-project-type) test-cockpit--project-types))
             (real-engine (funcall engine-factory)))
-      real-engine
+        real-engine
     (test-cockpit--make-dummy-engine)))
 
 (defmacro test-cockpit--engine-for-current-project ()
@@ -187,16 +193,20 @@ returned."
 (defun test-cockpit--retrieve-engine ()
   "Retrieve the engine valid for the current project.
 If no engine is yet started for the project, it will be started."
-  (if-let ((existing-engine (test-cockpit--engine-for-current-project)))
+  (if-let* ((existing-engine (test-cockpit--engine-for-current-project)))
       existing-engine
     (let ((new-engine (test-cockpit--make-engine)))
       (setf (test-cockpit--engine-for-current-project) new-engine)
       new-engine)))
 
+(defun test-cockpit--dummy-engine-p (&optional engine)
+  "Return non-nil if ENGINE or the current project enigne is a dummy engine."
+    (not (oref (or engine (test-cockpit--retrieve-engine)) project-type)))
+
 (defun test-cockpit--real-engine-or-error ()
   "Retrieve the engine for the project; error if the project type is unsupported."
   (let ((engine (test-cockpit--retrieve-engine)))
-    (if (oref engine is-dummy-engine)
+    (if (test-cockpit--dummy-engine-p engine)
         (signal "Project type %s not supported by test-cockpit or engine not installed"
                 (projectile-project-type))
       engine)))
@@ -215,14 +225,14 @@ trimmed string is then returned."
 The current module and function are determined automatically.
 The additional arguments are shipped as ARGS."
   (let ((engine (test-cockpit--retrieve-engine)))
-    (when-let ((last-module (test-cockpit--current-module-string)))
+    (when-let* ((last-module (test-cockpit--current-module-string)))
       (oset engine last-module-string last-module))
-    (when-let ((last-function (test-cockpit--current-function-string)))
+    (when-let* ((last-function (test-cockpit--current-function-string)))
       (oset engine last-function-string last-function))
     (oset engine last-args args)))
 
 (defun test-cockpit--update-last-interactive-command (function)
-  "Update thte last interactive command function"
+  "Update thte last interactive command function to FUNCTION."
   (let ((engine (test-cockpit--retrieve-engine)))
     (oset engine last-interactive-cmd function)))
 
@@ -248,15 +258,14 @@ FUNC-STRING is the string determining the function to test."
 
 (defun test-cockpit--infix ()
   "Call the infix function of the current project type and return the infix array."
-  (test-cockpit--transient-infix
-   (funcall (alist-get (projectile-project-type) test-cockpit--project-types))))
+  (test-cockpit--transient-infix (test-cockpit--retrieve-engine)))
 
 (defun test-cockpit--insert-infix ()
   "Insert the infix array into the `transient-prefix'."
   (unless (equal (aref (transient-get-suffix 'test-cockpit-prefix '(0)) 2)
                  '(:description "Run test"))
     (transient-remove-suffix 'test-cockpit-prefix '(0)))
-  (if-let (infix (test-cockpit--infix))
+  (if-let* ((infix (test-cockpit--infix)))
       (transient-insert-suffix 'test-cockpit-prefix '(0) infix)))
 
 (defun test-cockpit--run-test (command)
@@ -274,7 +283,7 @@ FUNC-STRING is the string determining the function to test."
 FUNC is the engine function responsible to setup the
 command.  THING is the thing to be tested.  ARGS the additional
 arguments."
-  (let ((command (funcall func thing args)))
+  (let ((command (funcall func thing (append args (test-cockpit--additional-switches)))))
     (oset (test-cockpit--retrieve-engine) last-switches args)
     command))
 
@@ -299,14 +308,15 @@ If we are not in a test module buffer, the last module we were in
 is tested."
   (interactive
    (list (transient-args 'test-cockpit-prefix)))
-  (if-let ((module-string (or (test-cockpit--current-module-string)
+  (if-let* ((module-string (or (test-cockpit--current-module-string)
                               (test-cockpit--last-module-string))))
-      (progn (test-cockpit--run-test
+      (progn
+        (test-cockpit--run-test
               (test-cockpit--command 'test-cockpit--make-test-module-command
                                      module-string
                                      args))
-             (test-cockpit--update-last-interactive-command 'test-cockpit-test-module)
-             (test-cockpit--update-last-commands args))
+        (test-cockpit--update-last-interactive-command 'test-cockpit-test-module)
+        (test-cockpit--update-last-commands args))
     (message "Not in a unit test module file")))
 
 ;;;###autoload
@@ -320,7 +330,7 @@ If we are not in a test function, the last module test function we
 were in is tested."
   (interactive
    (list (transient-args 'test-cockpit-prefix)))
-  (if-let ((function-string (or (test-cockpit--current-function-string)
+  (if-let* ((function-string (or (test-cockpit--current-function-string)
                                 (test-cockpit--last-function-string))))
       (progn (test-cockpit--run-test
               (test-cockpit--command 'test-cockpit--make-test-function-command
@@ -388,7 +398,7 @@ The command run is then stored in as last command of the project
 and thus can be repeated using `test-cockpit-repeat-test'."
   (interactive)
   (projectile-with-default-dir (projectile-project-root)
-    (when-let ((last-cmd (test-cockpit--last-custom-command)))
+    (when-let* ((last-cmd (test-cockpit--last-custom-command)))
       (setq compile-command last-cmd))
     (call-interactively #'compile))
   (oset (test-cockpit--retrieve-engine) last-custom-command compile-command)
@@ -426,10 +436,10 @@ If the for the project no test has been run during the current
 session, the main dispatch dialog is invoked."
   (interactive
    (list (transient-args 'test-cockpit-prefix)))
-  (if-let (last-cmd (oref (test-cockpit--real-engine-or-error) last-command))
+  (if-let* ((last-cmd (oref (test-cockpit--real-engine-or-error) last-command)))
       (if (eq last-cmd 'test-cockpit--last-command-was-dape)
           (test-cockpit-dape-debug-repeat-test)
-      (test-cockpit--run-test last-cmd))
+        (test-cockpit--run-test last-cmd))
     (test-cockpit-dispatch)))
 
 ;;;###autoload
@@ -439,7 +449,7 @@ If the project type is supported, function
 `test-cockpit-repeat-test' is run.  Otherwise the project build
 is launched by calling the function `projectile-compile-project'."
   (interactive)
-  (if (oref (test-cockpit--retrieve-engine) is-dummy-engine)
+  (if (test-cockpit--dummy-engine-p)
       (test-cockpit--projectile-build)
     (test-cockpit-dispatch)))
 
@@ -454,7 +464,7 @@ build command is repeated by the function
 `projectile-build-project' a prompt to type a build command is
 shown."
   (interactive)
-  (if (oref (test-cockpit--retrieve-engine) is-dummy-engine)
+  (if (test-cockpit--dummy-engine-p)
       (test-cockpit--repeat-projectile-build)
     (test-cockpit-repeat-test)))
 
@@ -465,7 +475,7 @@ If the project type is supported, function
 `test-cockpit-repeat-test' is run.  Otherwise the project tested
 calling the function `projectile-test-project'."
   (interactive)
-  (if (oref (test-cockpit--retrieve-engine) is-dummy-engine)
+  (if (test-cockpit--dummy-engine-p)
       (test-cockpit--projectile-test)
     (test-cockpit-dispatch)))
 
@@ -479,26 +489,25 @@ test menu is shown.  If the project type is unknown the last test
 command is repeated by the function `projectile-test-project' a
 prompt to type a test command is shown."
   (interactive)
-  (if (oref (test-cockpit--retrieve-engine) is-dummy-engine)
+  (if (test-cockpit--dummy-engine-p)
       (test-cockpit--repeat-projectile-test)
     (test-cockpit-repeat-test)))
 
 ;;;###autoload
-(defun test-cockpit--repeat-interactive-test (&optional args)
+(defun test-cockpit-repeat-interactive-test (&optional args)
   "Repeat the last interactive test command.
 This is not meant to be called directly but as a result the transient dispatch
 in order to call the last test action with modified ARGS."
   (interactive
    (list (transient-args 'test-cockpit-prefix)))
-  (when-let ((last-cmd (test-cockpit--last-interactive-test-command)))
+  (when-let* ((last-cmd (test-cockpit--last-interactive-test-command)))
     (funcall last-cmd args)))
 
 ;;;###autoload
 (defun test-cockpit-dape-debug-repeat-test ()
   "Repeat the last test action calling the dape debugger, if available."
   (interactive)
-  (if-let
-      ((config (test-cockpit--dape-debug-last-test)))
+  (if-let* ((config (test-cockpit--dape-debug-last-test)))
       (test-cockpit--launch-dape config)
     (user-error "No recent test-action has been performed or no Dape support for backend")))
 
@@ -541,7 +550,7 @@ description for the action."
 
 (defun test-cockpit--custom-actions ()
   "Make the transient suffix for the custom actions."
-  (when-let ((custom-actions
+  (when-let* ((custom-actions
               (alist-get (projectile-project-type) test-cockpit--project-type-custom-actions)))
     (vconcat ["Custom actions"] (vconcat custom-actions))))
 
@@ -646,11 +655,12 @@ repetition."
                                         `("d" "dape debug repeat" test-cockpit-dape-debug-repeat-test))
                                    ("c" "custom" test-cockpit-custom-test-command)
                                    ,(if last-cmd
-                                        `("r" "repeat" test-cockpit--repeat-interactive-test))))))))
+                                        `("r" ,(format "repeat %s" (test-cockpit--last-test-target))
+                                          test-cockpit-repeat-interactive-test))))))))
 
 (defun test-cockpit--main-suffix ()
   "Setup the main menu common for all projects for testing and actions."
-  (if-let ((custom-actions-suffix (test-cockpit--custom-actions)))
+  (if-let* ((custom-actions-suffix (test-cockpit--custom-actions)))
       `[,(test-cockpit--test-action-suffix) ,custom-actions-suffix]
     (test-cockpit--test-action-suffix)))
 
@@ -673,10 +683,18 @@ repetition."
                                               ,(format "last function: %s" (test-cockpit--strip-project-root function-string))
                                               test-cockpit--do-repeat-function)))))))))
 
+(defun test-cockpit--last-test-target ()
+  "Determine the semantics of the last test command."
+  (pcase (test-cockpit--last-interactive-test-command)
+    ('test-cockpit-test-project "project")
+    ('test-cockpit-test-module (format "module: %s" (test-cockpit--strip-project-root (test-cockpit--last-module-string))))
+    ('test-cockpit-test-function (format "function: %s" (test-cockpit--strip-project-root (test-cockpit--last-function-string))))
+    (_ "")))
+
 (defun test-cockpit--append-repeat-suffix ()
   "Append the repeat suffix to the transient prefix if possible.
 If possible the suffix is returned if not nil."
-  (if-let ((repeat-suffix (test-cockpit--transient-suffix-for-repeat)))
+  (if-let* ((repeat-suffix (test-cockpit--transient-suffix-for-repeat)))
       (transient-append-suffix 'test-cockpit-prefix '(-1) repeat-suffix)
     nil))
 
@@ -755,6 +773,17 @@ OJB is just the self reference."
                 (propertize ", " 'face 'transient-inactive-value))
      (propertize "]" 'face 'transient-inactive-value))))
 
+(defvar test-cockpit--additional-switch-config nil)
+
+(defun test-cockpit-add-additional-switch (project-type switch)
+  "Add the additional switch SWITCH to projects of PROJECT-TYPE."
+  (if-let* ((switch-list (alist-get project-type test-cockpit--additional-switch-config)))
+      (setcdr (assq project-type test-cockpit--additional-switch-config) (reverse (cons switch switch-list)))
+      (push (cons project-type (list switch)) test-cockpit--additional-switch-config)))
+
+(defun test-cockpit--additional-switches ()
+  "Lookup the additional switches for the current project type."
+  (alist-get (test-cockpit--primary-project-type) test-cockpit--additional-switch-config))
 
 (provide 'test-cockpit)
 
